==> fxt-2008.02.02.tgz: fxt/src/bmat/bitmat-hessenberg.cc <==

#include "bmat/bitmat-inline.h"

#include "bits/bitswap.h"
#include "aux0/swap.h"
#include "fxttypes.h"  // ulong

bool
bitmat_is_hessenberg(const ulong *H, ulong n)
// Return true if H is in Hessenberg form, else false.
{
    for (ulong k=2,m=1; k<n; ++k,m<<=1,m^=1)
    {
        if ( H[k] & m )  return false;
    }
    return true;
}
// -------------------------


static inline ulong bit_xor(ulong a, ulong k1, ulong k2)
// Return a with bit k1 XOR-ed to position k2.
// Used for adding columns in bit-matrices.
{
    ulong b = a >> k1;
    b &= 1;
    a ^= (b<<k2);
    return  a;
}
// -------------------------

// defines to make the code less cryptic:
#define mGET(M, i, j)  ((M[i]>>j) & 1UL)
//
void
bitmat_hessenberg(const ulong *M, ulong n, ulong *H)
// Set H to the Hessenberg form of M.
//.
// cf. Cohen p.55
{
    bitmat_copy(M, n, H);  // H=M

    for (ulong m=1; m<n-1; ++m)
    {
        // 2 [Search non-zero]:
        ulong i;
        for (i=m+1; i<n; ++i)  if ( mGET(H, i, m-1) )  break;
        if ( i==n )  continue;

        // t == 1;
        if ( i>m )
        {
            swap2(H[i], H[m]);
            for (ulong c=0; c<n; ++c)  H[c] = bit_swap(H[c], i, m);
        }

        // 3 [Eliminate]:
        for (i=m+1; i<n; ++i)
        {
            ulong q = mGET(H, i, m-1);
            if ( q )
            {
                // u = mGET(H, i, m-1)/t  == 1
                H[i] ^= H[m];
                for (ulong c=0; c<n; ++c)  H[c] = bit_xor(H[c], i, m);
            }
        }
    }
}
// -------------------------

==> fxt-2009.01.25.tgz: fxt/src/bmat/bitmat-hessenberg.cc <==

#include "bmat/bitmat-inline.h"

#include "bits/bitswap.h"
#include "aux0/swap.h"
#include "fxttypes.h"  // ulong

bool
bitmat_is_hessenberg(const ulong *H, ulong n)
// Return true if H is in Hessenberg form, else false.
{
    for (ulong k=2,m=1; k<n; ++k,m<<=1,m^=1)
    {
        if ( H[k] & m )  return false;
    }
    return true;
}
// -------------------------


static inline ulong bit_xor(ulong a, ulong k1, ulong k2)
// Return a with bit k1 XOR-ed to position k2.
// Used for adding columns in bit-matrices.
{
    ulong b = a >> k1;
    b &= 1;
    a ^= (b<<k2);
    return  a;
}
// -------------------------

// defines to make the code less cryptic:
#define mGET(M, i, j)  ((M[i]>>(j)) & 1UL)
//
void
bitmat_hessenberg(const ulong *M, ulong n, ulong *H)
// Set H to the Hessenberg form of M.
//.
// cf. Cohen p.55
{
    bitmat_copy(M, n, H);  // H=M

    for (ulong m=1; m<n-1; ++m)
    {
        // 2 [Search non-zero]:
        ulong i;
        for (i=m+1; i<n; ++i)  if ( mGET(H, i, m-1) )  break;
        if ( i==n )  continue;

        // t == 1;
        if ( i>m )
        {
            swap2(H[i], H[m]);
            for (ulong c=0; c<n; ++c)  H[c] = bit_swap(H[c], i, m);
        }

        // 3 [Eliminate]:
        for (i=m+1; i<n; ++i)
        {
            ulong q = mGET(H, i, m-1);
            if ( q )
            {
                // u = mGET(H, i, m-1)/t  == 1
                H[i] ^= H[m];
                for (ulong c=0; c<n; ++c)  H[c] = bit_xor(H[c], i, m);
            }
        }
    }
}
// -------------------------
